// This program is free software: you can redistribute it and/or modify 
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version. You can find the license alongside 
// this file in the release.

$(function() {
       
  // Global script value declaration START ----------------------------------------------------------------------------------------------

  // Used to figure out window numbers
  let zIndex = 0;
  
  // Used to be used to determine if the user is testing the site or not, you probably don't need to worry about this anymore
  let devMode = false;

  // Set this to false if your site doesn't have a .htaccess file, aka where all of your URLs have ".html" at the end 
  let hasHtaccess = true;

  // Set this to false if you are not using the Gullet log html block in your page 
  let hasGulletLog = true;

  // Used to know what page to load if the user is testing the site
  let urlToLoad = "";
  // Port at which python web service is hosted, default in included startserver.py is 8000
  const devServerPortNum = 8000;

  // Gullet context key:
  // 0 - undf - Undefined message type
  // 1 - burp - Successful action
  // 2 - gulp - Critical error
  // 3 - regurgitate - System message
  // 4 - belch - Startup message
  const gulletContext = ["undf ?? ", 
                          "burp -- ", 
                          "gulp !! ",
                          "regurgitate -- ",
                          "belch -- "];

  // Global script value declaration END ----------------------------------------------------------------------------------------------

  // Functions zone START ----------------------------------------------------------------------------------------------

  // Function to add a message to Gullet log window
  function gulletLog(message, context) {
    // Check if there is a Gullet log html block on the page 
    if (hasGulletLog) {
      // Add the message to the log window
      const $logContainer = $("#output");
      const $logLine = $("<div class='outputLine'></div>").text("Gullet : " + context + message);
      $logContainer.append($logLine);

      // Scroll to the bottom to show the latest message
      $logContainer.scrollTop($logContainer[0].scrollHeight);
    }
    // Log for in-built browser console
    console.log("Gullet : " + context + message);
  }

  // Lets Gullet say that it's alive if it has properly loaded, helps with debugging
  gulletLog("I'm alive!!",gulletContext[4])

  // Will recieve name of page to load and then send to createWindow to try and open it
  function gulletOpener(data){
    // console.log("Passed value:",data);
    createWindow(data);
  }

  // Used to get a manual user input and then will send that user input to gulletOpener
  function manualPageOpen(){
    // Get the user's input
    var userInput = prompt("Page name/folder?:");
    if (userInput !== null) {  // Check if the user pressed cancel;
      // Check if the user is using a .htaccess file, if they aren't, add .html to the end of the page name
      if (hasHtaccess) {
        gulletOpener(userInput);
      }
      else {
        gulletOpener(userInput + ".html")
      }
    }
  }

  // Used to create the Gullet windows themselves
  function createWindow(data) {

    // Notify that a new window is being created (helps with debugging).
    gulletLog("attempting to create window",gulletContext[3]);

    const $window = $("#windowTemplate").clone();
    $window.attr("id", "window" + zIndex);
    $window.find(".window-title").text(data);
    $window.css("z-index", zIndex);
    $window.appendTo("body");
    $window.show();

    // Make the new window draggable and resizable
    $window.draggable();
    $window.resizable(); 

    // Handle the close button click event
    $window.find(".close-button").click(function() {
      $window.remove();
      gulletLog("closed window",gulletContext[1]);
    });

    // Handle window click to bring it to the front
    $window.click(function() {
      $window.css("z-index", ++zIndex);
    });

    const $windowContent = $window.find(".window-content");

    // !! - Dynamic HTML loader START - !!

    // What is this?
    // This was implemented for my sanity when developing the site, in a nutshell, it checks for if the site is running locally 
    // (i.e. on localhost with the included python script) or if it is running on the actual site itself. Unless you're testing the 
    // site locally (e.g. by opening the html file locally in a web browser, not from starting my server script), you don't need to worry about this. Was also going to use dev mode for other things (e.g. more detailed error logs) but this never happened, keeping it here as to not accidentally break something.

    // Checks for if developer mode is enabled and chooses what should be loaded
    const htmlToLoad = data;      // Name of html page to load
    if(devMode){
      // If yes, load on localhost:<the port you chose>/<name of the page>
      urlToLoad = ("http://localhost:" + devServerPortNum + "/" + htmlToLoad); 
    }
    else{
      // Else, load file from server
      urlToLoad = htmlToLoad;
    }

    gulletLog("will try to load " + urlToLoad + " into window",gulletContext[3]);

    // The part that actually loads the page, thanks ajax
    $.ajax({
      url: urlToLoad,
      success: function(data) {
        $windowContent.html(data);
        gulletLog("loaded " + urlToLoad + " into window",gulletContext[1]);
      },
      error: function(error) {
        console.error("Error loading content:", error);
        if (devMode == false){
          $windowContent.html("Failed to load content. Is developer mode enabled?");
        }
        else{
          $windowContent.html("Failed to load content. Is the local dev server running? If you are a regular user, try disabling dev mode (Ctrl+.)");
        }
              
        gulletLog("failed to load html content in window",gulletContext[2]);
      }
    });

    // !! - Dynamic HTML loader END - !!

    zIndex++;

    // Confirm that the window has at least been created.
    gulletLog("created window " + zIndex,gulletContext[1]);
  }

  // Used to toggle developer mode
  function toggleDevMode(){
    devMode = !devMode;
  }

  // Functions zone END ----------------------------------------------------------------------------------------------

  // Quickbind zone START ----------------------------------------------------------------------------------------------

  // Quickbinds are essentially keyboard combinations that can be made while the page is open to perform a task with Gullet, e.g. opening the dialogue box with Ctrl + #, change these as you wish.

  // Toggle dev mode when "ctrl+." is pressed
  $(document).keydown(function(event) {
    if (event.ctrlKey && event.key === '.') {
      toggleDevMode();
      gulletLog("toggled developer mode to " + devMode,gulletContext[1]);
    }
  });

  // Open manual page goto when "ctrl+#" is pressed
  $(document).keydown(function(event) {
    if (event.ctrlKey && event.key === '#') {
      manualPageOpen();
    }
  });

  // Quickbind zone END ----------------------------------------------------------------------------------------------

  // Button declaration zone START ----------------------------------------------------------------------------------------------

  // If creating a GUI button to be used within the HTML, you'll need to state it's function here, these buttons can be seen on the example page above the GulletLog window. Make sure that the function names are the same as what you define in the html page, these buttons can be seen in the index.html example page at the start of the body section.

  // Clear Gullet log
  $("#clearGullet").click(function(){
    const $outputLine = $("#output");
    $outputLine.find(".outputLine").text("");
  });

  // Toggle developer mode
  $("#toggleDev").click(function(){
    toggleDevMode();
    gulletLog("toggled developer mode to " + devMode,gulletContext[1]);
  });

  // Check the current developer mode
  $("#checkDev").click(function(){
    if (devMode == true){
      gulletLog("developer mode is enabled",gulletContext[3]);
    }
    else {
      gulletLog("developer mode is disabled",gulletContext[3]);
    }
  });

  // Open the manual page open box
  $("#manualInput").click(function() {
    manualPageOpen();
  });

  // Button declaration zone END ----------------------------------------------------------------------------------------------

  // Page link zone START ----------------------------------------------------------------------------------------------

  // If creating a link in a html page to open a Gullet window, here's where you'll need to declare what page to open, as well as the name of the open function that you stated in the html page. These function names can be seen at the bottom of the index.html example page.

  // Open example page
  $("#openExample").click(function() {
    gulletOpener("example")
  });

  // Used to demonstrate how subfolders can be accessed
  $("#openNestedExample").click(function() {
    gulletOpener("example/nestedexample")
  });
  // Page link zone END ----------------------------------------------------------------------------------------------
});
